<?php

namespace IPTools;
use Brick\Math\BigInteger;
use Brick\Math\Exception\ArithmeticException;

/**
 * @author Safarov Alisher <alisher.safarov@outlook.com>
 * @link https://github.com/S1lentium/IPTools
 */
class Range implements \Iterator, \Countable
{
	use PropertyTrait;

	/**
	 * @var IP
	 */
	private $firstIP;
	/**
	 * @var IP
	 */
	private $lastIP;
	/**
	 * @var int
	 */
	private $position = 0;

	/**
	 * @param IP $firstIP
	 * @param IP $lastIP
	 * @throws \Exception
	 */
	public function __construct(IP $firstIP, IP $lastIP)
	{		
		$this->setFirstIP($firstIP);
		$this->setLastIP($lastIP);
	}

    /**
     *
     * @return string
     */
    public function __toString()
    {
        return $this->getFirstIP().'-'.$this->getLastIP();
    }

	/**
	 * @param string $data
	 * @return Range
	 */
	public static function parse($data)
	{
		if($data instanceof Range){
		    return clone $data;
        } elseif (strpos($data,'/') || strpos($data,' ')) {
			$network = Network::parse($data);
			$firstIP = $network->getFirstIP();
			$lastIP  = $network->getLastIP();
		} elseif (strpos($data, '*')) {
			$firstIP = IP::parse(str_replace('*', '0', $data));
			$lastIP  = IP::parse(str_replace('*', '255', $data));
		} elseif (strpos($data, '-')) {
			list($first, $last) = explode('-', $data, 2);
			$firstIP = IP::parse($first);
			$lastIP  = IP::parse($last);
		} else {
			$firstIP = IP::parse($data);
			$lastIP  = clone $firstIP;
		}

		return new self($firstIP, $lastIP);
	}

	/**
	 * @param IP|Network|Range $find
	 * @return bool
	 * @throws \Exception
	 */
	public function contains($find)
	{
		if ($find instanceof IP) {
			$within = (strcmp($find->inAddr(), $this->firstIP->inAddr()) >= 0)
				&& (strcmp($find->inAddr(), $this->lastIP->inAddr()) <= 0);
		} elseif ($find instanceof Range || $find instanceof Network) {
			/**
			 * @var Network|Range $find
			 */
			$within = (strcmp($find->getFirstIP()->inAddr(), $this->firstIP->inAddr()) >= 0)
				&& (strcmp($find->getLastIP()->inAddr(), $this->lastIP->inAddr()) <= 0);
		} else {
            $within = $this->contains(self::parse($find));
		}

		return $within;
	}

	/**
	 * @param IP $ip	 
	 * @throws \Exception
	 */
	public function setFirstIP(IP $ip)
	{
		if ($this->lastIP && strcmp($ip->inAddr(), $this->lastIP->inAddr()) > 0) {
			throw new \Exception('First IP is grater than second');
		}

		$this->firstIP = $ip;
	}

	/**
	 * @param IP $ip	 
	 * @throws \Exception
	 */
	public function setLastIP(IP $ip)
	{
		if ($this->firstIP && strcmp($ip->inAddr(), $this->firstIP->inAddr()) < 0) {
			throw new \Exception('Last IP is less than first');
		}

		$this->lastIP = $ip;
	}

	/**
	 * @return IP
	 */
	public function getFirstIP()
	{
		return $this->firstIP;
	}

	/**
	 * @return IP
	 */
	public function getLastIP()
	{
		return $this->lastIP;
	}

    /**
     * @return array
     * @throws \Exception
     */
	public function getNetworks()
	{
		$span = $this->getSpanNetwork();

		$networks = array();

		if ($span->getFirstIP()->inAddr() === $this->firstIP->inAddr()
			&& $span->getLastIP()->inAddr() === $this->lastIP->inAddr()
		) {
			$networks = array($span);
		} else {
			if ($span->getFirstIP()->inAddr() !== $this->firstIP->inAddr()) {
				$excluded = $span->exclude($this->firstIP->prev());

				/**
				 * @var Network $network
				 */
				foreach ($excluded as $network) {
					if (strcmp($network->getFirstIP()->inAddr(), $this->firstIP->inAddr()) >= 0) {
						$networks[] = $network;
					}
				}
			}

			if ($span->getLastIP()->inAddr() !== $this->lastIP->inAddr()) {
				if (!$networks) {
					$excluded = $span->exclude($this->lastIP->next());
				} else {
					$excluded = array_pop($networks);
					$excluded = $excluded->exclude($this->lastIP->next());
				}

				foreach ($excluded as $network) {
					$networks[] = $network;
					if ($network->getLastIP()->inAddr() === $this->lastIP->inAddr()) {
						break;
					}
				}
			}

		}

		return $networks;
	}

	/**
	 * @return Network
     * @throws \Exception
	 */
	public function getSpanNetwork()
	{
		$xorIP = IP::parseInAddr($this->getFirstIP()->inAddr() ^ $this->getLastIP()->inAddr());

		preg_match('/^(0*)/', $xorIP->toBin(), $match);

		$prefixLength = strlen($match[1]);

		$ip = IP::parseBin(str_pad(substr($this->getFirstIP()->toBin(), 0, $prefixLength), $xorIP->getMaxPrefixLength(), '0'));

		return new Network($ip, Network::prefix2netmask($prefixLength, $ip->getVersion()));
	}

    /**
     * Exclude range of IP from
     *
     * @param $exclude
     * @return Range[]
     * @throws \Exception
     */
	public function exclude($exclude){
        $ranges = [clone $this];

        foreach ((array) $exclude as $excludeRng) {
            $excludeRng = self::parse($excludeRng);

            foreach ($ranges as $k => $range) {
                /** @var $range Range */

                if ( $range->contains($excludeRng)){
                    if(strcmp($excludeRng->getLastIP()->next()->inAddr(), $range->getLastIP()->inAddr()) <= 0){
                        $ranges[] = new Range($excludeRng->getLastIP()->next(), $range->getLastIP());
                    }

                    if(strcmp($range->getFirstIP()->inAddr(), $excludeRng->getFirstIP()->prev(1)->inAddr()) <= 0){
                        $range->setLastIP($excludeRng->getFirstIP()->prev(1));
                    }else{
                        unset($ranges[$k]);
                    }
                }elseif ($range->contains($excludeRng->getFirstIP())) {
                    $trimIp = $excludeRng->getFirstIP()->prev(1);
                    if ($range->contains($trimIp))
                        $range->setLastIP($trimIp);

                } elseif ($range->contains($excludeRng->getLastIP())) {
                    $trimIp = $excludeRng->getLastIP()->next(1);
                    if ($range->contains($trimIp))
                        $range->setFirstIP($trimIp);

                } elseif ($excludeRng->contains($range)){
                    unset($ranges[$k]);
                }
            }
        }

        return $ranges;
    }

    /**
     * @return BigInteger
     */
    public function getRangeSize()
    {
        return BigInteger::of($this->lastIP->toLong())
            ->minus($this->firstIP->toLong())
            ->plus(1);
    }

    /**
     * @return IP
     * @throws \Exception
     */
	public function current()
	{
		return $this->firstIP->next($this->position);
	}

	/**
	 * @return int
	 */
	public function key()
	{
		return $this->position;
	}

	public function next()
	{
		++$this->position;
	}

	public function rewind()
	{
		$this->position = 0;
	}

    /**
     * @return bool
     * @throws \Exception
     */
	public function valid()
	{
		return strcmp($this->firstIP->next($this->position)->inAddr(), $this->lastIP->inAddr()) <= 0;
	}

	/**
	 * @return int
     * @throws ArithmeticException
	 */
	public function count()
	{
        return $this->getRangeSize()->toInteger();
	}

}
